#ifndef CYGONCE_HAL_PLATFORM_SETUP_H
#define CYGONCE_HAL_PLATFORM_SETUP_H

/*=============================================================================
//
//      hal_platform_setup.h
//
//      Platform specific support for HAL (assembly code)
//
//=============================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
// Copyright (C) 2006 eCosCentric Limited 
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):   sg
// Contributors:
// Date:        2006-07-19
// Purpose:     LPC-H2294 platform specific support routines
// Description: 
// Usage:       #include <cyg/hal/hal_platform_setup.h>
//
//####DESCRIPTIONEND####
//
//===========================================================================*/

#include <pkgconf/system.h>
#include <cyg/hal/var_io.h>

        // There is alone red LED on the board
        // |P0.30 470 Om     LED
        // |------\/\/\/-----|<|-----> 3.3 V
        // |
        .macro  _led_init
        ldr r0,=CYGARC_HAL_LPC2XXX_REG_IO_BASE
        ldr r1,[r0,#CYGARC_HAL_LPC2XXX_REG_IO0DIR]
        orr r1,r1,#(1<<30)
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_IO0DIR]
        
        .endm // _led_init

        .macro _led x
        ldr r0,=CYGARC_HAL_LPC2XXX_REG_IO_BASE
        ldr r1,=(1<<30)
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_IO0SET]
        ldr r1,=((\x & 1)<<30)
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_IO0CLR]
        
        .endm // _led
        
        .macro _pll_init
        ldr r0,=CYGARC_HAL_LPC2XXX_REG_SCB_BASE
        mov r2,#0xAA
        mov r3,#0x55
        // enable PLL
        mov r1,#1
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_PLLCON]
        mov r1,#(0x20 | (CYGNUM_HAL_ARM_LPC2XXX_PLL_MUL - 1))
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_PLLCFG]
        // update PLL registers
        str r2,[r0,#CYGARC_HAL_LPC2XXX_REG_PLLFEED]
        str r3,[r0,#CYGARC_HAL_LPC2XXX_REG_PLLFEED]
        // wait for it to lock
1:
        ldr r1,[r0,#CYGARC_HAL_LPC2XXX_REG_PLLSTAT]
        ands r1,r1,#(1<<10)
        beq 1b
        // connect PLL
        mov r1,#3
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_PLLCON]
        // update PLL registers
        str r2,[r0,#CYGARC_HAL_LPC2XXX_REG_PLLFEED]
        str r3,[r0,#CYGARC_HAL_LPC2XXX_REG_PLLFEED]
        
        .endm // _pll_init
        
        .macro _mem_init
        // copy first 64 bytes of rom/flash to RAM
        mov r0,#0
        mov r1,#0x40000000
        mov r2,#0x40
1:
        ldr r3,[r0,#4]!
        str r3,[r1,#4]!
        cmps r0,r2
        bne 1b

        ldr r0,=CYGARC_HAL_LPC2XXX_REG_SCB_BASE
        // interrupt vector table is mapped to:
        mov r1,#2 // 1 -- flash, 2 -- RAM
        str r1, [r0,#CYGARC_HAL_LPC2XXX_REG_MEMMAP]
        // flash timings
        mov r1,#4
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_MAMTIM]
        mov r1,#2 // 2 -- full MAM
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_MAMCR]

        // BCFG0 (an external FLASH memory, Intel TF28F320C3 x 1)
        // 16-bits; IDCY = F; WST1 = 4; RBLE = 1; WST2 = 6
        ldr r0,=CYGARC_HAL_LPC2XXX_REG_BCFG0
        ldr r1,=0x1000348f
        str r1,[r0]

        // BCFG1 (an external SRAM IDT71V416L x 2)
        // 32-bits; IDCY = F; WST1 = 0; RBLE = 1; WST2 = 0
        ldr r0,=CYGARC_HAL_LPC2XXX_REG_BCFG1
        ldr r1,=0x2000040f
        str r1,[r0]

        .endm // _mem_init

        .macro _gpio_init
        ldr r0,=CYGARC_HAL_LPC2XXX_REG_PIN_BASE

        // configure GPIO0's TxD0, RxD0 for UART0
        ldr r1,=0x00000005
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_PINSEL0]

        // configure GPIO1 for general purpose I/O
        ldr r1,=0
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_PINSEL1]

        // it's more magic
        ldr r1,=0x0F000924
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_PINSEL2]
                
        // configure GPIO0/GPIO1's pinouts for input
        ldr r1,=0
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_IO0DIR]
        ldr r1,=0
        str r1,[r0,#CYGARC_HAL_LPC2XXX_REG_IO1DIR]

        .endm // _gpio_init
        
#ifdef CYGHWR_HAL_ARM_LPCH2294_DIAG_LEDS
#define CYGHWR_LED_MACRO _led \x 
#endif

#if defined(CYG_HAL_STARTUP_ROM)

        .macro  _setup
        
        _pll_init
        
        _mem_init
        
        _gpio_init
        
        _led_init

        .endm

#define CYGSEM_HAL_ROM_RESET_USES_JUMP

#else

        .macro  _setup

        .endm
        
#endif

#define PLATFORM_SETUP1     _setup

#endif // CYGONCE_HAL_PLATFORM_SETUP_H
